@@ -18,11 +18,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
-#include "exif-entry.h"
-#include "exif-ifd.h"
-#include "exif-utils.h"
-#include "i18n.h"
+#include <config.h>
+
+#include <libexif/exif-entry.h>
+#include <libexif/exif-ifd.h>
+#include <libexif/exif-utils.h>
+#include <libexif/i18n.h>
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -31,9 +32,6 @@
 #include <time.h>
 #include <math.h>
 
-#undef  MIN
-#define MIN(a, b)  (((a) < (b)) ? (a) : (b))
-
 struct _ExifEntryPrivate
 {
 	unsigned int ref_count;
@@ -159,6 +157,8 @@ exif_entry_fix (ExifEntry *e)
 {
 	unsigned int i;
 	ExifByteOrder o;
+	ExifRational r;
+	ExifSRational sr;
 
 	if (!e || !e->priv) return;
 
@@ -199,9 +199,9 @@ exif_entry_fix (ExifEntry *e)
 				exif_format_get_size (e->format);
 			e->data = exif_entry_realloc (e, e->data, e->size);
 			exif_entry_log (e, EXIF_LOG_CODE_DEBUG,
-				"Tag '%s' was of format '%s' (which is "
+				_("Tag '%s' was of format '%s' (which is "
 				"against specification) and has been "
-				"changed to format '%s'.",
+				"changed to format '%s'."),
 				exif_tag_get_name (e->tag), 
 				exif_format_get_name (EXIF_FORMAT_LONG),
 				exif_format_get_name (EXIF_FORMAT_SHORT));
@@ -212,28 +212,54 @@ exif_entry_fix (ExifEntry *e)
 		}
 		break;
 
+	/* All these tags need to be of format 'Rational'. */
+	case EXIF_TAG_FNUMBER:
+	case EXIF_TAG_APERTURE_VALUE:
+	case EXIF_TAG_EXPOSURE_TIME:
+	case EXIF_TAG_FOCAL_LENGTH:
+		switch (e->format) {
+		case EXIF_FORMAT_SRATIONAL:
+			if (!e->parent || !e->parent->parent) break;
+			o = exif_data_get_byte_order (e->parent->parent);
+			for (i = 0; i < e->components; i++) {
+				sr = exif_get_srational (e->data + i * 
+					exif_format_get_size (
+						EXIF_FORMAT_SRATIONAL), o);
+				r.numerator = (ExifLong) sr.numerator;
+				r.denominator = (ExifLong) sr.denominator;
+				exif_set_rational (e->data + i *
+					exif_format_get_size (
+						EXIF_FORMAT_RATIONAL), o, r);
+			}
+			e->format = EXIF_FORMAT_RATIONAL;
+			exif_entry_log (e, EXIF_LOG_CODE_DEBUG,
+				_("Tag '%s' was of format '%s' (which is "
+				"against specification) and has been "
+				"changed to format '%s'."),
+				exif_tag_get_name (e->tag),
+				exif_format_get_name (EXIF_FORMAT_SRATIONAL),
+				exif_format_get_name (EXIF_FORMAT_RATIONAL));
+			break;
+		default:
+			break;
+		}
+		break;
+
 	case EXIF_TAG_USER_COMMENT:
 
 		/* Format needs to be UNDEFINED. */
 		if (e->format != EXIF_FORMAT_UNDEFINED) {
 			exif_entry_log (e, EXIF_LOG_CODE_DEBUG,
-				"Tag 'UserComment' had invalid format '%s'. "
-				"Format has been set to 'undefined'.",
+				_("Tag 'UserComment' had invalid format '%s'. "
+				"Format has been set to 'undefined'."),
 				exif_format_get_name (e->format));
 			e->format = EXIF_FORMAT_UNDEFINED;
 		}
 
-		/* Some cameras fill the tag with '\0' or ' '. */
-		for (i = 0; i < e->size &&
-			    (!e->data[i] || (e->data[i] == ' ')); i++);
-		if (i && (i == e->size)) {
-			exif_entry_log (e, EXIF_LOG_CODE_DEBUG,
-				"Tag 'UserComment' contained unnecessary "
-				"data which has been removed.");
-			exif_mem_free (e->priv->mem, e->data);
-			e->data = NULL;
-			e->size = 0;
-			e->components = 0;
+		/* Some packages like Canon ZoomBrowser EX 4.5 store
+		   only one zero byte followed by 7 bytes of rubbish */
+		if ((e->size >= 8) && (e->data[0] == 0)) {
+			memcpy(e->data, "\0\0\0\0\0\0\0\0", 8);
 		}
 
 		/* There need to be at least 8 bytes. */
@@ -248,10 +274,12 @@ exif_entry_fix (ExifEntry *e)
 			/* Assume ASCII */
 			memmove (e->data + 8, e->data, e->size);
 			memcpy (e->data, "ASCII\0\0\0", 8);
+			e->size += 8;
+			e->components += 8;
 			exif_entry_log (e, EXIF_LOG_CODE_DEBUG,
-				"Tag 'UserComment' has been expanded to at "
+				_("Tag 'UserComment' has been expanded to at "
 				"least 8 bytes in order to follow the "
-				"specification.");
+				"specification."));
 			break;
 		}
 
@@ -260,17 +288,24 @@ exif_entry_fix (ExifEntry *e)
 		 * afterwards, let's assume ASCII and claim the 8 first
 		 * bytes for the format specifyer.
 		 */
-		if (i >= 8) {
+		for (i = 0; (i < e->size) && !e->data[i]; i++);
+		if (!i) for ( ; (i < e->size) && (e->data[i] == ' '); i++);
+		if ((i >= 8) && (i < e->size)) {
 			exif_entry_log (e, EXIF_LOG_CODE_DEBUG,
-				"Tag 'UserComment' did not start with "
+				"Tag 'UserComment' is not empty but does not start with "
 				"format identifyer. This has been fixed.");
 			memcpy (e->data, "ASCII\0\0\0", 8);
+			break;
 		}
 
-		/* First 8 bytes need to follow the specification. */
-		if (memcmp (e->data, "ASCII\0\0\0"  , 8) &&
-		    memcmp (e->data, "UNICODE\0"    , 8) &&
-		    memcmp (e->data, "JIS\0\0\0\0\0", 8)) {
+		/* 
+		 * First 8 bytes need to follow the specification. If they don't, 
+		 * assume ASCII.
+		 */
+		if (memcmp (e->data, "ASCII\0\0\0"     , 8) &&
+		    memcmp (e->data, "UNICODE\0"       , 8) &&
+		    memcmp (e->data, "JIS\0\0\0\0\0"   , 8) &&
+		    memcmp (e->data, "\0\0\0\0\0\0\0\0", 8)) {
 			e->data = exif_entry_realloc (e, e->data, 8 + e->size);
 			if (!e->data) {
 				e->size = 0;
@@ -281,9 +316,11 @@ exif_entry_fix (ExifEntry *e)
 			/* Assume ASCII */
 			memmove (e->data + 8, e->data, e->size);
 			memcpy (e->data, "ASCII\0\0\0", 8);
+			e->size += 8;
+			e->components += 8;
 			exif_entry_log (e, EXIF_LOG_CODE_DEBUG,
-				"Tag 'UserComment' did not start with "
-				"format identifyer. This has been fixed.");
+				_("Tag 'UserComment' did not start with "
+				"format identifier. This has been fixed."));
 			break;
 		}
 
@@ -397,6 +434,7 @@ static struct {
       {  0, {NULL}}}},
   { EXIF_TAG_COMPRESSION,
     { {1, {N_("Uncompressed"), NULL}},
+      {5, {N_("LZW compression"), NULL}},
       {6, {N_("JPEG compression"), NULL}},
       {0, {NULL}}}},
   { EXIF_TAG_LIGHT_SOURCE,
@@ -437,7 +475,7 @@ static struct {
       {4, {N_("Shutter priority"),N_("Shutter"), NULL}},
       {5, {N_("Creative program (biased toward depth of field)"),
 	   N_("Creative"), NULL}},
-      {6, {N_("Creative program (biased toward fast shutter speed"),
+      {6, {N_("Creative program (biased toward fast shutter speed)"),
 	   N_("Action"), NULL}},
       {7, {N_("Portrait mode (for closeup photos with the background out "
 	      "of focus)"), N_("Portrait"), NULL}},
@@ -488,6 +526,7 @@ static struct {
       {0, {NULL}}}},
   { EXIF_TAG_COLOR_SPACE,
     { {1, {N_("sRGB"), NULL}},
+      {2, {N_("Adobe RGB"), NULL}},
       {0xffff, {N_("Uncalibrated"), NULL}}}},
   {0, }
 };
@@ -495,7 +534,8 @@ static struct {
 const char *
 exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 {
-	unsigned int i, j, k, l;
+	unsigned int i, j, k, l, ts;
+	const unsigned char *t;
 	ExifByte v_byte;
 	ExifShort v_short, v_short2, v_short3, v_short4;
 	ExifLong v_long;
@@ -520,12 +560,18 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		{NULL  , 0,  0}
 	};
 
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	bindtextdomain (GETTEXT_PACKAGE, LIBEXIF_LOCALEDIR);
+	/* FIXME: This belongs to somewhere else. */
+	/* libexif should use the default system locale.
+	 * If an application specifically requires UTF-8, then we
+	 * must give the application a way to tell libexif that.
+	 * 
+	 * bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	 */
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 
 	/* We need the byte order */
 	if (!e || !e->parent || !e->parent->parent)
-		return NULL;
+		return val;
 	o = exif_data_get_byte_order (e->parent->parent);
 
 	memset (val, 0, maxlen);
@@ -563,7 +609,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		 * NULL terminated.
 		 */
 		if ((e->size >= 8) && !memcmp (e->data, "ASCII\0\0\0", 8)) {
-			strncpy (val, e->data + 8, MIN (e->size - 8, maxlen));
+			strncpy (val, (char *) e->data + 8, MIN (e->size - 8, maxlen));
 			break;
 		}
 		if ((e->size >= 8) && !memcmp (e->data, "UNICODE\0", 8)) {
@@ -586,11 +632,11 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		 * Print as much as possible.
 		 */
 		exif_entry_log (e, EXIF_LOG_CODE_DEBUG,
-			"Tag UserComment does not comply "
-			"with standard but contains data.");
+			_("Tag UserComment does not comply "
+			"with standard but contains data."));
 		for (; (i < e->size)  && (strlen (val) < maxlen - 1); i++) {
 			exif_entry_log (e, EXIF_LOG_CODE_DEBUG,
-				"Byte at position %i: 0x%02x", i, e->data[i]);
+				_("Byte at position %i: 0x%02x"), i, e->data[i]);
 			val[strlen (val)] =
 				isprint (e->data[i]) ? e->data[i] : '.';
 		}
@@ -628,8 +674,8 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		 * Some cameras store a string like "   " here. Ignore it.
 		 */
 		if (e->size && e->data &&
-		    (strspn (e->data, " ") != strlen ((char *) e->data)))
-			strncpy (val, e->data, MIN (maxlen, e->size));
+		    (strspn ((char *)e->data, " ") != strlen ((char *) e->data)))
+			strncpy (val, (char *) e->data, MIN (maxlen, e->size));
 		else
 			strncpy (val, _("[None]"), maxlen);
 		strncat (val, " ", maxlen - strlen (val));
@@ -637,13 +683,14 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 
 		/* Second part: Editor. */
 		strncat (val, " - ", maxlen - strlen (val));
-		if (e->size && e->data &&
-		    (strlen ((char *) e->data) + 1 < e->size) &&
-		    (strspn (e->data, " ") != strlen ((char *) e->data)))
-			strncat (val, e->data + strlen (e->data) + 1,
-				 maxlen - strlen (val));
-		else
+		if (e->size && e->data) {
+		    t = e->data + strlen ((char *) e->data) + 1;
+		    ts = e->data + e->size - t;
+		    if ((ts > 0) && (strspn ((char *)t, " ") != ts))
+			strncat (val, (char *)t, MIN (maxlen - strlen (val), ts));
+		} else {
 			strncat (val, _("[None]"), maxlen - strlen (val));
+		}
 		strncat (val, " ", maxlen - strlen (val));
 		strncat (val, _("(Editor)"), maxlen - strlen (val));
 
@@ -652,7 +699,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		CF (e, EXIF_FORMAT_RATIONAL, val, maxlen);
 		CC (e, 1, val, maxlen);
 		v_rat = exif_get_rational (e->data, o);
-		if (!v_rat.denominator) return NULL;
+		if (!v_rat.denominator) return val;
 		snprintf (val, maxlen, "f/%.01f", (float) v_rat.numerator /
 						    (float) v_rat.denominator);
 		break;
@@ -660,7 +707,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		CF (e, EXIF_FORMAT_RATIONAL, val, maxlen);
 		CC (e, 1, val, maxlen);
 		v_rat = exif_get_rational (e->data, o);
-		if (!v_rat.denominator) return NULL;
+		if (!v_rat.denominator) return val;
 		snprintf (val, maxlen, "f/%.01f",
 			  pow (2 , ((float) v_rat.numerator /
 				    (float) v_rat.denominator) / 2.));
@@ -669,7 +716,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		CF (e, EXIF_FORMAT_RATIONAL, val, maxlen);
 		CC (e, 1, val, maxlen);
 		v_rat = exif_get_rational (e->data, o);
-		if (!v_rat.denominator) return NULL;
+		if (!v_rat.denominator) return val;
 
 		/*
 		 * For calculation of the 35mm equivalent,
@@ -680,14 +727,14 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		entry = exif_content_get_entry (
 			e->parent->parent->ifd[EXIF_IFD_0], EXIF_TAG_MAKE);
 		if (entry && entry->data &&
-		    !strncmp (entry->data, "Minolta", 7)) {
+		    !strncmp ((char *)entry->data, "Minolta", 7)) {
 			entry = exif_content_get_entry (
 					e->parent->parent->ifd[EXIF_IFD_0],
 					EXIF_TAG_MODEL);
 			if (entry && entry->data) {
-				if (!strncmp (entry->data, "DiMAGE 7", 8))
+				if (!strncmp ((char *)entry->data, "DiMAGE 7", 8))
 					d = 3.9;
-				else if (!strncmp (entry->data, "DiMAGE 5", 8))
+				else if (!strncmp ((char *)entry->data, "DiMAGE 5", 8))
 					d = 4.9;
 			}
 		}
@@ -705,7 +752,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		CF (e, EXIF_FORMAT_RATIONAL, val, maxlen);
 		CC (e, 1, val, maxlen);
 		v_rat = exif_get_rational (e->data, o);
-		if (!v_rat.denominator) return NULL;
+		if (!v_rat.denominator) return val;
 		snprintf (val, maxlen, "%.1f m", (float) v_rat.numerator /
 						   (float) v_rat.denominator);
 		break;
@@ -713,25 +760,25 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		CF (e, EXIF_FORMAT_RATIONAL, val, maxlen);
 		CC (e, 1, val, maxlen);
 		v_rat = exif_get_rational (e->data, o);
-		if (!v_rat.denominator) return NULL;
+		if (!v_rat.denominator) return val;
 		d = (double) v_rat.numerator / (double) v_rat.denominator;
 		if (d < 1)
 			snprintf (val, maxlen, _("1/%d"),
 				  (int) (1. / d));
 		else
 			snprintf (val, maxlen, _("%d"), (int) d);
-		if (maxlen > strlen (val) + strlen (" sec."))
-			strncat (val, " sec.", maxlen - strlen (val) - 1);
+		if (maxlen > strlen (val) + strlen (_(" sec.")))
+			strncat (val, _(" sec."), maxlen - strlen (val) - 1);
 		break;
 	case EXIF_TAG_SHUTTER_SPEED_VALUE:
 		CF (e, EXIF_FORMAT_SRATIONAL, val, maxlen);
 		CC (e, 1, val, maxlen);
 		v_srat = exif_get_srational (e->data, o);
-		if (!v_srat.denominator) return NULL;
+		if (!v_srat.denominator) return val;
 		snprintf (val, maxlen, "%.0f/%.0f", (float) v_srat.numerator,
 			  (float) v_srat.denominator);
-		if (maxlen > strlen (val) + strlen (" sec."))
-			strncat (val, " sec.", maxlen - strlen (val) - 1);
+		if (maxlen > strlen (val) + strlen (_(" sec.")))
+			strncat (val, _(" sec."), maxlen - strlen (val) - 1);
 		snprintf (b, sizeof (b), " (APEX: %i)",
 			(int) pow (sqrt(2), (float) v_srat.numerator /
 				            (float) v_srat.denominator));
@@ -776,8 +823,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		CF (e, EXIF_FORMAT_SRATIONAL, val, maxlen);
 		CC (e, 1, val, maxlen);
 		v_srat = exif_get_srational (e->data, o);
-		if (!v_srat.denominator)
-			return NULL;
+		if (!v_srat.denominator) return val;
 		snprintf (val, maxlen, "%s%.01f",
 			  v_srat.denominator * v_srat.numerator > 0 ? "+" : "",
 			  (double) v_srat.numerator /
@@ -851,16 +897,16 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		/* Search the tag */
 		for (i = 0; list2[i].tag && (list2[i].tag != e->tag); i++);
 		if (!list2[i].tag) {
-			strncpy (val, "Internal error.", maxlen - 1);
+			strncpy (val, _("Internal error."), maxlen - 1);
 			break;
 		}
 
 		/* Find the value */
 		for (j = 0; list2[i].elem[j].values &&
 			    (list2[i].elem[j].index < v_short); j++);
-		if (list2[i].elem[j].index != v_short) {
-			snprintf (val, maxlen, "Internal error (unknown "
-				  "value %i).", v_short);
+		if (list2[i].elem[j].index != v_short || v_short == 65535) {
+			snprintf (val, maxlen, _("Internal error (unknown "
+				  "value %i)."), v_short);
 			break;
 		}
 
@@ -895,7 +941,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 		/* Search the tag */
 		for (i = 0; list[i].tag && (list[i].tag != e->tag); i++);
 		if (!list[i].tag) {
-			strncpy (val, "Internal error.", maxlen - 1);
+			strncpy (val, _("Internal error."), maxlen - 1);
 			break;
 		}
 
@@ -941,7 +987,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 			break;
 		case EXIF_FORMAT_LONG:
 			v_long = exif_get_long (e->data, o);
-			snprintf (val, maxlen, "%i", (int) v_long);
+			snprintf (val, maxlen, "%li", (long int) v_long);
 			maxlen -= strlen (val);
 			for (i = 1; i < e->components; i++) {
 				v_long = exif_get_long (e->data +
@@ -970,7 +1016,7 @@ exif_entry_get_value (ExifEntry *e, char *val, unsigned int maxlen)
 			}
 			break;
 		case EXIF_FORMAT_ASCII:
-			strncpy (val, e->data, MIN (maxlen, e->size));
+			strncpy (val, (char *) e->data, MIN (maxlen, e->size));
 			break;
 		case EXIF_FORMAT_RATIONAL:
 			v_rat = exif_get_rational (e->data, o);
@@ -1242,21 +1288,21 @@ exif_entry_initialize (ExifEntry *e, ExifTag tag)
 	case EXIF_TAG_MODEL:
 	case EXIF_TAG_SOFTWARE:
 	case EXIF_TAG_ARTIST:
-		e->components = strlen ("[None]") + 1;
+		e->components = strlen (_("[None]")) + 1;
 		e->format = EXIF_FORMAT_ASCII;
 		e->size = exif_format_get_size (e->format) * e->components;
 		e->data = exif_entry_alloc (e, e->size);
 		if (!e->data) break;
-		strncpy (e->data, "[None]", e->size);
+		strncpy ((char *)e->data, _("[None]"), e->size);
 		break;
 	case EXIF_TAG_COPYRIGHT:
-		e->components = (strlen ("[None]") + 1) * 2;
+		e->components = (strlen (_("[None]")) + 1) * 2;
 		e->format = EXIF_FORMAT_ASCII;
 		e->size = exif_format_get_size (e->format) * e->components;
 		e->data = exif_entry_alloc (e, e->size);
 		if (!e->data) break;
-		strcpy (e->data +                     0, "[None]");
-		strcpy (e->data + strlen ("[None]") + 1, "[None]");
+		strcpy (((char *)e->data) + 0, _("[None]"));
+		strcpy (((char *)e->data) + strlen (_("[None]")) + 1, _("[None]"));
 		break;
 
 	/* UNDEFINED, no components, no default */
