@@ -50,8 +50,14 @@ exif_format_get_name (ExifFormat format)
 {
 	unsigned int i;
 
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	bindtextdomain (GETTEXT_PACKAGE, LIBEXIF_LOCALEDIR);
+	/* FIXME: This belongs to somewhere else. */
+	/* libexif should use the default system locale.
+	 * If an application specifically requires UTF-8, then we
+	 * must give the application a way to tell libexif that.
+	 * 
+	 * bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	 */
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 
 	for (i = 0; ExifFormatTable[i].name; i++)
 		if (ExifFormatTable[i].format == format)
